mainDir<-"E:/study/yan/supprting of senstivity of Vegetation/Irtysh River"
setwd(mainDir)
library("raster")


####The converted data is used and renamed NDVI
######Convert data format######
tranf<-function(var='NDVI',divider=10000){
dir <- paste(mainDir, "/", var, sep="")   #Main directory
dirNew <- 	paste(mainDir, "/", var,"_ori", sep="")
dir.create(dirNew)     
filenames <- list.files(path= dir, pattern =".txt")  
progBar <- txtProgressBar(min = 0, max =length(filenames) , style = 3)   #length(filenames)

for(k in 1: length(filenames)){
  
  #Read in the data
  fullData <- read.table(paste(dir, "/", filenames[k], sep=""), header=FALSE)
  
  #Split data set
  final_data=matrix()
  
  for(i in 1:1460){
    for(j in 1:1533){
      final_data[1533*(i-1)+j]<-fullData[i,j]/divider    
    }
  }
  write.table(final_data, 
              file=paste(dirNew, "/",filenames[k], ".txt", sep=""),
              sep=",", 
              col.names=FALSE, row.names= FALSE)
  setTxtProgressBar(progBar, k)
}
}

#tranf(var='NDVI',divider=10000)
#tranf(var='Tem',divider=100)
#tranf(var='Pre',divider=100)
#tranf(var='TVDI',divider=1000)





# Import the landsea mask and plot it
landsea<-read.table("landsea.txt", header=F)
landsea<-c(as.matrix(landsea))
length(landsea)

landsea.matrix <- matrix(landsea, nrow=1460, byrow=TRUE)
lsRaster <- raster(landsea.matrix)
#plot(lsRaster)


mask<-c(which(landsea==1))
length(mask)

# Need this landsea.tile.matrix for later on in the script
landsea.tile.matrix <- matrix(landsea, ncol= 10, byrow = FALSE)

##########Split dataset###########
maketiles<-function(var='NDVI'){
######  first split the data for each variable up into 10 'tiles' due to large size of files.
dir <- paste(mainDir, "/", var, sep="")  
dirNew <- 	paste(mainDir, "/", var,"_tiled", sep="")
dir.create(dirNew)      

fileNames <- list.files(path= dir, pattern =".txt")  
nTiles <- 10        
tileMatrix <- matrix(c(1:2238180), ncol= nTiles, byrow=FALSE) # Make a matrix used to subset the different tiles in the variable

#bar
print("Splitting data into tiles")
progBar <- txtProgressBar(min = 0, max = length(fileNames), style = 3)

###filenames
for(k in 1: length(fileNames)){
  varName <- unlist(strsplit(fileNames[k], split=".", fixed=TRUE))[1]  
  dirName <- paste(mainDir,"/", var, "_tiled/", varName, sep="")   
  # make a new directory for each month for the variable of interest
  dir.create(dirName) 
  
  #Read in the data
  fullData <- read.table(paste(dir, "/", fileNames[k], sep=""), header=FALSE)[,1]		
  
  # For each of the 100 tiles grap the observations from the tile of interest (defined within tile matrix) and then save in the new directory
  for(i in 1: nTiles){
    dataToGet <- tileMatrix[,i]	
    tiledData <- fullData[dataToGet]
    write.table(tiledData, 
                file=paste(dirName, "/",varName, "_", i, ".csv", sep=""), 
                sep=",", 
                col.names=FALSE, row.names= FALSE
    )
    
  } # End of nTiles loop[i]
  
  setTxtProgressBar(progBar, k)
} # End of fileNames [k] loop
}
#maketiles(var='NDVI')
#maketiles(var='Tem')
#maketiles(var='Pre')
maketiles(var='TVDI')



#########################################################################################
## Sort all the tiled data into a new  directory, (eg. evi_sorted)
#########################################################################################
sorttiles<-function(var='NDVI'){
dir <- paste(mainDir, "/", var, "_tiled", sep="") # Directory with the tiled data from above function
fileNamesMonth <- list.files(path= dir)

dirNew <- paste(mainDir, "/", var, "_sorted", sep="")
dir.create(dirNew)

print("Moving data into tiles")
progBar <- txtProgressBar(min = 0, max = 10, style = 3)

for( j in 1: 10){
  tileName <- paste(var, "_", j, sep="") # Name for directory for tile 
  dir.create(paste(dirNew,"/", tileName, sep="")) # Create a new directory for the file	
  
  for(i in 1:length(fileNamesMonth)){
    
    fileToMove <- paste(dir,"/", fileNamesMonth[i],"/", fileNamesMonth[i], "_", j, ".csv", sep="") 
    fileToPlace <- paste(dirNew, "/", tileName, "/",  fileNamesMonth[i], "_", j, ".csv", sep="")
    file.rename(from = fileToMove, to = fileToPlace)
  }
  setTxtProgressBar(progBar, j)
}
unlink(dir, recursive = TRUE) # Deletes directory where the moved files were placed
}
nTiles=10

#sorttiles(var='NDVI')
#sorttiles(var='Tem')
#sorttiles(var='Pre')
sorttiles(var='TVDI')

##########################################################################################
## PREPARE DATA FUNCTION TO MAKE TIMESERIES TILE BY TILE.
## NB. ONLY DOING THIS FOR LAND AREAS TO SAVE COMPUTiNG TIME LATER, SO USE 'landsea.tile.matrix' OBJECT TO DO THIS
###########################################################################################
prepareData <- function (var = "NDVI", tile = 1,mask.matrix = landsea.tile.matrix){
  
  newDir <-  paste(mainDir, "/", var, "_ts", sep="")
  dir.create(newDir)
  
  #Import each monthly tiled image, mask it and find the land areas in the specific tile from the mask
  tilePath <- paste(mainDir, "/", var,"_sorted/", var, "_", tile, sep="")
  filenames<-list.files(path=tilePath, pattern=".csv") # reads the filenames in the directory
  
  mask.full <- mask.matrix[,tile] # select which column in the matrix you are interested in 
  mask <- which(mask.full!= -9999) # select the land areas
  
  resultMatrix <- matrix(NA, ncol=length(filenames), nrow= length(mask), dimnames= list(seq(1:length(mask)), filenames))  # Months/ years will be in columns, pixels from the landsea mask will be in rows
  setwd(tilePath)
  
  print("Compiling data into time series within a tile")
  progBar <- txtProgressBar(min = 0, max = length(filenames), style = 3)
  
  for(j in 1: length(filenames)){
    
    data1<-read.csv(file=filenames[j], header=F) 
    data1<-c(as.matrix(data1))
    resultMatrix[,j]<-data1[mask]
    setTxtProgressBar(progBar, j)
  }
  write.csv(resultMatrix, file= paste(newDir, "/", var, "_ts_", tile,".csv", sep=""))
}

# THEN USE THE PREPARE DATA FUNCTION TO MAKE THE TIME SERIES FOR EACH MONTH (OK to ignore warnings when this function completed)

#for(i in 1: 10) prepareData(var = "NDVI", tile =i,mask.matrix = landsea.tile.matrix)
#for(i in 1: 10) prepareData(var = "Tem", tile =i,mask.matrix = landsea.tile.matrix)
#for(i in 1: 10) prepareData(var = "Pre", tile =i,mask.matrix = landsea.tile.matrix)
for(i in 1: 10) prepareData(var = "TVDI", tile =i,mask.matrix = landsea.tile.matrix)








